﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using System.IO;

namespace Primeval
{
    static class GenePool
    {
        /// <summary>
        /// Building a list of the bots used in the current generation
        /// </summary>
        private static List<Bot> generation;

        /// <summary>
        /// Bots from the last generation, sorted by efficacy
        /// </summary>
        private static List<Bot> lastGeneration;

        /// <summary>
        /// The number of bots to make each generation
        /// </summary>
        public const int generationSize = 7;

        public static int generationCount;

        public static int GenerationSize { get { return generation.Count; } }

        public static bool GenerationOver { get { return generation.Count >= generationSize; } }

        public static void Reset()
        {
            generation = new List<Bot>();
            lastGeneration = null;
            generationCount = 0;
        }

        static GenePool()
        {
            generation = new List<Bot>();

            frame = new Rectangle(
                694, 50, 300, 170);
            textframe = new Rectangle(
                frame.X + 25, frame.Y + 20, frame.Width - 35, frame.Height - 30);
        }

        public static void LoadContent()
        {
            laser = Game1.content.Load<Texture2D>("laser");
            gun = Game1.content.Load<Texture2D>("bullet");
            rocket = Game1.content.Load<Texture2D>("rocket");
        }

        /// <summary>
        /// Finalize the current generation and prepare for the next
        /// </summary>
        public static void NextGeneration()
        {
            lastGeneration = generation;
            lastGeneration.Sort();

            generationCount++;

            generation = new List<Bot>();
        }

        /// <summary>
        /// Spawn a new bot
        /// </summary>
        public static void SpawnBot()
        {
            if (lastGeneration == null)
                generation.Add(new Bot().Spawn());
            else
                generation.Add(
                    new Bot().Spawn(lastGeneration[0].Genome.Cross(lastGeneration[1].Genome).Mutate()));
        }

        public static void AddSeedBot(Genome genome)
        {
            if (lastGeneration == null) lastGeneration = new List<Bot>();
            lastGeneration.Add(new Bot().Spawn(genome));
        }

        /// <summary>
        /// Spawn a bot from the given genome
        /// </summary>
        /// <param name="genome"></param>
        public static void SpawnBot(Genome genome)
        {
            generation.Add(new Bot().Spawn(genome));
        }

        /// <summary>
        /// Save out the current bot
        /// </summary>
        public static bool SaveBot()
        {
            if (generation.Count == 0) return false;

            if (!Directory.Exists("Exports")) Directory.CreateDirectory("Exports");

            Bot bot = generation[generation.Count - 1];

            string finfile;
            if (File.Exists("Exports/" + bot.Genome.name + ".product"))
            {
                int c = 0;
                string file = "Exports/" + bot.Genome.name + " " + MessageDisplay.PadNum(c, 4) + ".product";
                while (File.Exists(file))
                {
                    c++;
                    file = "Exports/" + bot.Genome.name + " " + MessageDisplay.PadNum(c, 4) + ".product";
                }
                finfile = file;
            }
            else
                finfile = "Exports/" + bot.Genome.name + ".product";
            bot.Genome.Write(finfile);

            return true;
        }

        private static Rectangle frame;
        private static Rectangle textframe;

        private const int lineheight = 30;

        private static Texture2D laser;
        private static Texture2D rocket;
        private static Texture2D gun;

        /// <summary>
        /// Draw a record of this generation to the screen
        /// </summary>
        /// <param name="spriteBatch"></param>
        public static void Draw(SpriteBatch spriteBatch)
        {
            //Draw caption
            spriteBatch.DrawString(
                Game1.readoutFont,
                "Products",
                new Vector2(
                    frame.Center.X - Game1.readoutFont.MeasureString("Products").X / 2, 20),
                Color.White);

            //Draw background box
            spriteBatch.Draw(
                Game1.pixel,
                new Rectangle(frame.X, frame.Y, frame.Width, lineheight * 2),
                new Color(0.8f, 1, 0.8f, 0.3f));
            spriteBatch.Draw(
                Game1.pixel,
                new Rectangle(frame.X, frame.Y + lineheight * 2,
                    frame.Width, frame.Height - lineheight * 2),
                new Color(0.8f, 0.8f, 0.8f, 0.3f));

            //Draw bots
            for (int c = 0; c < 5; c++)
            {
                Bot bot;
                if (c < 2)
                {
                    if (lastGeneration != null)
                        bot = lastGeneration[c];
                    else
                        continue;
                }
                else if (generation.Count > c - 2)
                    bot = generation[c - 2 + Math.Max(0, generation.Count - 3)];
                else
                    break;

                int ypos = textframe.Y + c * lineheight;

                //Frame
                spriteBatch.Draw(
                    Bot.frame,
                    new Vector2(textframe.X, ypos),
                    null,
                    bot.color,
                    0f,
                    new Vector2(Bot.frame.Width, Bot.frame.Height) / 2,
                    bot.radius * 2 / 600f,
                    SpriteEffects.None,
                    1f);

                //Weapons
                for (int d = 0; d < bot.weapons.Count; d++)
                {
                    if (bot.weapons[d] is Laser)
                    {
                        spriteBatch.Draw(
                            laser,
                            new Rectangle(
                                textframe.X + 30 + d * 30,
                                ypos - lineheight / 2,
                                (int)((Laser)bot.weapons[d]).radius,
                                lineheight),
                            ((Laser)bot.weapons[d]).color);
                    }
                    else if (bot.weapons[d] is RocketLauncher)
                    {
                        spriteBatch.Draw(
                            rocket,
                            new Vector2(textframe.X + 30 + d * 30, ypos),
                            null,
                            Color.White,
                            (float)((RocketLauncher)bot.weapons[d]).RocketDirection,
                            new Vector2(rocket.Width, rocket.Height) / 2,
                            ((RocketLauncher)bot.weapons[d]).Radius / 40f,
                            SpriteEffects.None,
                            1f);
                    }
                    else if (bot.weapons[d] is Gun)
                    {
                        int xpos = textframe.X + 30 + d * 30 - 15;
                        for (int e = 0; e < ((Gun)bot.weapons[d]).SpreadCount; e++)
                        {
                            spriteBatch.Draw(
                                gun,
                                new Vector2(xpos, ypos),
                                null,
                                Color.White,
                                0,
                                new Vector2(gun.Width, gun.Height) / 2,
                                ((Gun)bot.weapons[d]).Radius / gun.Width,
                                SpriteEffects.None,
                                1f);
                            xpos += 8;
                        }
                    }
                }
            }
        }
    }
}
