﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace Primeval
{
    enum ParticleType
    {
        SMOKE, FLAME, FRAG1, FRAG2, FRAG3, SHARD, SPARK
    }

    class Particle
    {
        #region Static
        private static List<Particle> particles;

        static Particle()
        {
            particles = new List<Particle>();
        }

        public static void Add(Particle p)
        {
            particles.Add(p);
        }

        public static void Reset()
        {
            particles.Clear();
        }

        public static void UpdateAll(GameTime gameTime)
        {
            for (int c = particles.Count - 1; c >= 0; c--)
            {
                particles[c].Update(gameTime);
                if (particles[c].Die) particles.RemoveAt(c);
            }
        }

        public static void DrawAll(SpriteBatch spriteBatch)
        {
            for (int c = particles.Count - 1; c >= 0; c--) particles[c].Draw(spriteBatch);
        }
        #endregion

        private TimeSpan lifeTime;
        private TimeSpan totalLifeTime;

        private bool fadeInOut;

        public bool Die { get { return lifeTime <= TimeSpan.Zero; } }

        private ParticleType type;

        private Vector2 position;
        private Vector2 velocity;
        private Vector2 acceleration;
        private float rotation;
        private float rotationVelocity;

        private Color color = Color.White;

        private bool spawns = false;
        private ParticleType spawn;
        private TimeSpan spawnInterval;
        private TimeSpan currentSpawnInterval;

        private float scale = 1;

        private static Texture2D[] image;

        public Particle(ParticleType type, Vector2 position, TimeSpan lifeTime)
        {
            this.type = type;
            this.position = position;
            this.totalLifeTime = lifeTime;
            this.lifeTime = totalLifeTime;
            rotation = (float)(Game1.random.NextDouble() * Math.PI * 2);
            rotationVelocity = (float)(Game1.random.NextDouble() - 0.5) * 3;
            velocity = new Vector2(
                (float)Game1.random.NextDouble() * 25,
                (float)Game1.random.NextDouble() * 25);
            Particle.Add(this);
        }

        public Particle(ParticleType type, Vector2 position, TimeSpan lifeTime, TimeSpan initLife)
            : this(type, position, lifeTime)
        {
            this.lifeTime = initLife;
        }

        public Particle(ParticleType type, Vector2 position, Vector2 velocity,
            Vector2 gravity, TimeSpan lifeTime)
            : this (type, position, lifeTime)
        {
            this.velocity = velocity;
            this.acceleration = gravity;
        }

        public Particle(ParticleType type, Vector2 position, Vector2 velocity,
            Vector2 gravity, TimeSpan lifeTime, float spin, Color color)
            : this(type, position, velocity, gravity, lifeTime)
        {
            rotationVelocity = spin;
            this.color = color;
        }

        public Particle(ParticleType type, Vector2 position, Vector2 velocity,
            Vector2 gravity, TimeSpan lifeTime, float spin, Color color,
            ParticleType spawns, TimeSpan interval, float scale)
            : this(type, position, velocity, gravity, lifeTime, spin, color)
        {
            this.scale = scale;
            this.spawns = true;
            spawn = spawns;
            spawnInterval = interval;
        }

        public Particle(ParticleType type, Vector2 position, Vector2 velocity,
            Vector2 gravity, TimeSpan lifeTime, bool fadeInOut)
            : this(type, position, velocity, gravity, lifeTime)
        {
            this.fadeInOut = fadeInOut;
        }

        public static void LoadContent()
        {
            image = new Texture2D[7];
            image[0] = Game1.content.Load<Texture2D>("smoke");
            image[1] = Game1.content.Load<Texture2D>("flame");
            image[2] = Game1.content.Load<Texture2D>("frag1");
            image[3] = Game1.content.Load<Texture2D>("frag2");
            image[4] = Game1.content.Load<Texture2D>("frag3");
            image[5] = Game1.content.Load<Texture2D>("shard");
            image[6] = Game1.content.Load<Texture2D>("spark");
        }

        public void Update(GameTime gameTime)
        {
            lifeTime -= gameTime.ElapsedGameTime;

            rotation += rotationVelocity * (float)gameTime.ElapsedGameTime.TotalSeconds;
            velocity += acceleration *(float)gameTime.ElapsedGameTime.TotalSeconds;
            position += velocity * (float)gameTime.ElapsedGameTime.TotalSeconds;

            currentSpawnInterval -= gameTime.ElapsedGameTime;
            if (currentSpawnInterval <= TimeSpan.Zero && spawns)
            {
                new Particle(
                    spawn,
                    position,
                    TimeSpan.FromSeconds(1));
                currentSpawnInterval = spawnInterval;
            }
        }

        public void Draw(SpriteBatch spriteBatch)
        {
            //Determine alpha to use
            float alpha;
            if (fadeInOut)
            {
                if (lifeTime.TotalSeconds < totalLifeTime.TotalSeconds / 2)
                {
                    alpha = (float)(lifeTime.TotalSeconds / (totalLifeTime.TotalSeconds / 2));
                }
                else
                {
                    alpha = 1 - (float)(
                        (lifeTime.TotalSeconds - totalLifeTime.TotalSeconds / 2)
                        / (totalLifeTime.TotalSeconds / 2));
                }
            }
            else
            {
                alpha = (float)(lifeTime.TotalSeconds / totalLifeTime.TotalSeconds);
            }

            spriteBatch.Draw(
                image[(int)type],
                Game1.TransformToDraw(position),
                null,
                new Color(color.R, color.G, color.B, (byte)(255 * alpha)),
                rotation,
                new Vector2(image[(int)type].Width, image[(int)type].Height) / 2,
                scale,
                SpriteEffects.None,
                1f);
        }
    }
}
